/*******************************************************
 *
 * $Id: FlatshadeScreen.cpp,v 1.5 2003/05/31 19:54:08 teacy Exp $
 *
 * This file has been placed in the public domain.
 *
 *******************************************************/


#include <PalmOS.h>

#include "FlatshadeScreen.h"
#include "gfx/Display.h"
#include "gfx/Canvas.h"


#define project(v,z) ((int)(((((long)(v)) << 5) / ((z)+32))))


FlatshadeScreen::FlatshadeScreen(const StateDescriptor *screenState) : Screen()
{
	state = (FlatshadeScreenState *)(screenState->state);
	exitScreen = false;
}


void FlatshadeScreen::init(const Canvas *, const Canvas *)
{
	beginTime  = TimGetTicks();
	frameCount = 0L;
}


FlatshadeScreen::~FlatshadeScreen()
{
}



	
Boolean FlatshadeScreen::nextPeriod()
{
	frameCount++;

	state->periodCounter++;
	return exitScreen;
}


void FlatshadeScreen::getViewportProperties(const Canvas *displayCanvas, ViewportProperties& viewport) const
{
	viewport.region.topLeft.x =  0;
	viewport.region.topLeft.y = 30;
	viewport.region.extent.x = displayCanvas->width;
	viewport.region.extent.y = displayCanvas->height - 60;
}


void FlatshadeScreen::drawBackground(const Canvas *viewportCanvas) const
{
	viewportCanvas->fillRectangle(Color::WHITE, 0, 0, viewportCanvas->width - 1, viewportCanvas->height - 1);
}


void FlatshadeScreen::drawAction(const Canvas *viewportCanvas, RectangleType *bounds)
{
	static const Color orange = Color(250, 160, 40);
	static const Color cyan = Color(255, 128, 255);
	static const Color yellow = Color(255, 255, 128);

	viewportCanvas->fillRectangle(Color::WHITE, 0, 0, viewportCanvas->width - 1, viewportCanvas->height - 1);

	Int16 modifier = (state->periodCounter & 31) << 2;

	if (modifier > 64)
	{
		modifier = 128 - modifier;
	}

	xy e4[4];
	e4[0].x = 20 + modifier;
	e4[0].y = 20 - modifier;
	e4[1].x = 97 + modifier;
	e4[1].y = 37 + modifier;
	e4[2].x = 108;
	e4[2].y = 130;
	e4[3].x = -30 + modifier;
	e4[3].y = 141;
	
	viewportCanvas->drawPolygon(e4, 4, Color::BLACK);

	e4[0].x = 97 + modifier;
	e4[0].y = 37 + modifier;
	e4[1].x = 188;
	e4[1].y = 41;
	e4[2].x = 194;
	e4[2].y = 132;
	e4[3].x = 108;
	e4[3].y = 130;
	
	viewportCanvas->drawPolygon(e4, 4, orange);

	xy e3[3];
	e3[0].x = -30 + modifier;
	e3[0].y = 141;
	e3[1].x = 108;
	e3[1].y = 130;
	e3[2].x = 90;
	e3[2].y = 180 - modifier;
	
	viewportCanvas->drawPolygon(e3, 3, orange);

	Boolean color = true;
	const int y = 100 + modifier;
	const int yOffset = 60;
	for (int x = -200; x < 200; x += 50)
	{
		for (int z = 200; z >= 20; z -= 20)
		{
			e4[0].x = project(x, z) + 80;
			e4[0].y = project(y, z) + yOffset;
			e4[1].x = project(x + 50, z) + 80;
			e4[1].y = project(y, z) + yOffset;
			e4[2].x = project(x + 50, z-20) + 80;
			e4[2].y = project(y, z-20) + yOffset;
			e4[3].x = project(x, z-20) + 80;
			e4[3].y = project(y, z-20) + yOffset;
			
			viewportCanvas->drawPolygon(e4, 4, (color) ? cyan : yellow);
			color = !color;
		}
		color = !color;
	}	


	bounds->topLeft.x = 0;
	bounds->topLeft.y = 0;
	bounds->extent.x = viewportCanvas->width;
	bounds->extent.y = viewportCanvas->height;
}


void FlatshadeScreen::drawFixedOverlays(const Canvas *displayCanvas) const
{
	displayCanvas->fillRectangle(Color::BLACK, 0, 0, displayCanvas->width - 1, 29);
	displayCanvas->fillRectangle(Color::BLACK, 0, displayCanvas->height - 30, displayCanvas->width - 1, displayCanvas->height - 1);
}


void FlatshadeScreen::drawDynamicOverlays(const Canvas *displayCanvas) const
{
	const UInt32 elapsedTime = (TimGetTicks() - beginTime) + 1;
	UInt32 fps = (frameCount * 1000) / elapsedTime;
	Char frameChar[50];
	StrPrintF(frameChar, "%3lu /10 fps", fps);
	displayCanvas->drawText(Color::WHITE, Color::BLACK, stdFont, frameChar, 11, 1, 10);
}


void FlatshadeScreen::receiveEvent(const EventType* evtPtr)
{
	if (evtPtr->eType == penUpEvent)
	{
		exitScreen = true;
	}
}


StateDescriptor *FlatshadeScreen::createStateDescriptor()
{	
	createState(FlatshadeScreenState);
	return stateDescriptor;
}


bool FlatshadeScreen::enableMenu() const
{
    return false;
}

