/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Display.cpp,v 1.4 2003/05/30 12:30:20 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


/*
 TODO: Should dirty rectangle management become a function of the Canvas?
 */


#include "./HostControl.h"
#include <PalmOS.h>

#include "gfx/Display.h"
#include "gfx/Canvas.h"
#include "gfx/Geometry.h"
#include "Device.h"


Display::Display() :
    displayCanvas(NULL), viewportCanvas(NULL)
{
}


Display::~Display() 
{
	// This cannot invoke destroy(), because destructor in baseclass may not involve pure virtual functions
	// in subclasses.
}


void Display::destroy()
{
	if (viewportCanvas != NULL)
	{
		delete viewportCanvas;
	}

	if (displayCanvas != NULL)
	{
		delete displayCanvas;
	}

	resetDisplay();	
}


void Display::beginDrawViewport()
{
	viewportCanvas->setOSCanvas();

	lastBounds.topLeft.x = currentBounds.topLeft.x;
	lastBounds.topLeft.y = currentBounds.topLeft.y;
	lastBounds.extent.x = currentBounds.extent.x;
	lastBounds.extent.y = currentBounds.extent.y;
}
	

void Display::endDrawViewport(RectangleType *bounds)
{
	if (bounds == NULL)
	{
		currentBounds.topLeft.x = 0;
		currentBounds.topLeft.y = 0;
		currentBounds.extent.x = viewport.region.extent.x;
		currentBounds.extent.y = viewport.region.extent.y;
	}
	else
	{	
		currentBounds.topLeft.x = bounds->topLeft.x;
		currentBounds.topLeft.y = bounds->topLeft.y;
		currentBounds.extent.x = bounds->extent.x;
		currentBounds.extent.y = bounds->extent.y;

		// Clip bounds of changed region by viewportRegion (only vertical clipping is currently supported)
		if (currentBounds.extent.y != 0)
		{
			if (currentBounds.topLeft.y >= viewport.region.extent.y)
			{
				// Bounding box is below viewport
				currentBounds.topLeft.y = currentBounds.extent.y = 0;
			}
			else if ((currentBounds.topLeft.y + currentBounds.extent.y) <= 0)
			{
				// Bounding box is above viewport
				currentBounds.topLeft.y = currentBounds.extent.y = 0;
			}
			else if (currentBounds.topLeft.y < 0)
			{
				// Top edge of bounding box is above top edge of viewport
				Coord yDiff = -currentBounds.topLeft.y;
				currentBounds.extent.y -= yDiff;
				currentBounds.topLeft.y = 0;
			}				
			
			if ((currentBounds.topLeft.y + currentBounds.extent.y) > viewport.region.extent.y)
			{
				// Bottom edge of bounding box is below bottom edge of viewport
				Coord yDiff = (currentBounds.topLeft.y + currentBounds.extent.y) - viewport.region.extent.y;
				currentBounds.extent.y -= yDiff;
			}
		}
		else
		{
			currentBounds.topLeft.y = 0;
		}
	}

	Geometry::uniteBounds(&lastBounds, &currentBounds, &copyBounds);


	displayCanvas->setOSCanvas();
}




void Display::initViewport(const ViewportProperties& viewport)
{
	// Destroy previous viewport
	if (viewportCanvas != NULL)
	{
		delete viewportCanvas;
	}

	createViewportCanvas(viewport);	

	// Remember new viewport
	MemMove(&this->viewport, &viewport, sizeof(ViewportProperties));

	// Prepare bounds for screen copy
	currentBounds.topLeft.x = 0;
	currentBounds.topLeft.y = 0;
	currentBounds.extent.x = viewport.region.extent.x;
	currentBounds.extent.y = viewport.region.extent.y;
}


Canvas *Display::getDisplayCanvas() const
{
	return displayCanvas;
}

Canvas *Display::getViewportCanvas() const
{
	return viewportCanvas;
}


UInt16 Display::rows;
UInt16 Display::cols;

UInt32 Display::depth;
Display::Density Display::density;
Boolean Display::colorMode;
Boolean Display::directColor;
