/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Flatshader.cpp,v 1.2 2003/05/29 15:45:25 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#include <PalmOS.h>

#include "Customization.h"

#include "gfx/Display.h"
#include "gfx/Canvas.h"
#include "gfx/Flatshader.h"
#include "gfx/FlatshaderArmlet.h"
#include "Device.h"
#include "Armlet.h"

#ifdef USE_SDK50
const Armlet Flatshader::flatshaderArmlet = Armlet(1000, NULL /* "Flatshader.dll\0NativeFunction" */);
#endif

void Flatshader::drawPolygon(const Canvas *canvas, const xy *e, UInt16 vertexCount, const Color color)
{
	int shiftFactor = canvas->shiftFactor;
	int	clipyo = canvas->rows - 1;
	int clipxr = canvas->cols - 1;

	// TODO: This wastes memory on 160x160 screens, and won't work on 320x480 Hires+
	Int16 minx[320],maxx[320];

#ifdef USE_SDK50
		if (flatshaderArmlet.isAvailable() && (Display::depth == 8))
		{
			FlatShaderParamType shaderParams;
			shaderParams.minxptr = &minx[0];
			shaderParams.maxxptr = &maxx[0];
			shaderParams.rowBytes = canvas->rowBytes;
			shaderParams.drawBits = (UInt8 *)canvas->drawBits;
			shaderParams.clipxr = clipxr;
			shaderParams.clipyo = clipyo;
			shaderParams.colorIdx = color.getIndexedColor();
			shaderParams.vertexCount = vertexCount;
			shaderParams.e = (void *)e;
			shaderParams.shiftFactor = shiftFactor;
			flatshaderArmlet.invoke(&shaderParams);
			return;
		}
#endif

	// Determine the miny/maxy
	int miny, maxy;
	miny = maxy = e[0].y << shiftFactor;
	
	for(UInt16 vertexIdx = 1; vertexIdx < vertexCount; vertexIdx++)
	{
		int vy = e[vertexIdx].y << shiftFactor;
		if (vy < miny)
		{
			miny = vy;
		}		
		else if (vy > maxy)
		{
			maxy = vy;
		}
	}

	if (miny < 0)
	{
		miny = 0;
	}
	else if (miny > clipyo)
	{
		return;
	}
	
	if (maxy > clipyo)
	{
		maxy = clipyo;
	}
	else if (maxy < 0)
	{
		return;
	}
	

	// Initialize the minx/maxx fields
	for(int y = miny; y <= maxy; y++)
	{
		minx[y] = 32000;
		maxx[y] = 0;
	}
	
	// Iterate over the edges of the polygon
	UInt16 vertexIdx = vertexCount;
	int flago=0, flagu=0;
	
	do 
	{
		UInt16 vIdx = (vertexIdx == vertexCount) ? 0 : vertexIdx;

		int y1 = e[vIdx].y << shiftFactor;
		int x1 = e[vIdx].x << shiftFactor;

		int y2 = e[vertexIdx-1].y << shiftFactor;
		int x2 = e[vertexIdx-1].x << shiftFactor;

		// horizontal edge
		if (y1 == y2) 
		{
			if (y1 > clipyo) { flago++; continue; }
			if (y1 < 0) { flagu++; continue; }
			if (x1 > x2) { int swap = x1; x1 = x2; x2 = swap; }
			if (x1 < 0) x1 = 0;
			if (x1 < minx[y1]) minx[y1] = x1;

			if (x2 > clipxr) x2 = clipxr;
			if (x2 > maxx[y1]) maxx[y1] = x2;
		}
		// vertical edge
		else if (x1 == x2)
		{
			if (x1 < 0) 
			{
				x1 = 0;
			}
			else if (x1 > clipxr) 
			{
				x1 = clipxr;
			}

			if (y1 > y2) { int yswap = y1; y1 = y2; y2 = yswap; }
			if (y1 > clipyo) {flago++; continue;} // Trivial rejection
			if (y2 < 0) {flagu++; continue;} // Trivial rejection
			if (y1 < 0) y1 = 0;
			if (y2 > clipyo) y2 = clipyo;
			
			int rowcount = y2-y1;
			Int16 *minxptr = &minx[y1];
			Int16 *maxxptr = &maxx[y1];
			do 
			{
				if (x1 < *minxptr) *minxptr = x1;
				if (x1 > *maxxptr) *maxxptr = x1;
				minxptr++;
				maxxptr++;
			} while(--rowcount >= 0);
		}
		// diagonal edge
		else 
		{
			// Proprietary clipping algorithm
			if (y1 > y2)
				{ int swap = x1; x1 = x2; x2 = swap;
					swap = y1; y1 = y2; y2 = swap; }
			if (y1 > clipyo) {flago++; continue;} // Trivial rejection
			if (y2 < 0) {flagu++; continue;} // Trivial rejection

			if (y1 < 0) 
			{
				long m = ((x2-x1) << 7) / (y2-y1);
				x1 -= (int)((m * y1) >> 7);
				y1 = 0;
			}
			if (y2 > clipyo) 
			{
				long m = (long(x2-x1) << 7) / (y2-y1);
				x2 += (int)((m * (clipyo-y2)) >> 7);
				y2 = clipyo;
			}

			if (y1 == y2) 
			{
				// Edge has degenerated into a horizontal edge b/c of clipping
				if (x1 < 0) 
				{
					x1 = 0;
				}
				else if (x1 > clipxr) 
				{
					x1 = clipxr;
				}
				if (x1 < minx[y1]) minx[y1] = x1;
				if (x1 > maxx[y1]) maxx[y1] = x1;

				if (x2 < 0) 
				{
					x2 = 0;
				}
				else if (x2 > clipxr) 
				{
					x2 = clipxr;
				}
				if (x2 < minx[y1]) minx[y1] = x2;
				if (x2 > maxx[y1]) maxx[y1] = x2;
				continue;
			}

			// Scanline conversion of the edge (Bresenham)
			{
				long m = (long(x2 - x1) << 7) / (y2-y1); 
				long cx = 0;

				Int16 *minxptr = &minx[y1];
				Int16 *maxxptr = &maxx[y1];

				int rowcount = y2-y1;				
				do 
				{
					int x = x1 + ((int)(cx >> 7));
					if (x < 0) 
					{
						x = 0;
					}
					else if (x > clipxr) 
					{
						x = clipxr;
					}
					if (x < *minxptr) *minxptr = x;
					if (x > *maxxptr) *maxxptr = x;
					
					cx += m;
					minxptr++;
					maxxptr++;
				} while (--rowcount >= 0);
			}
		}
	} while(--vertexIdx);

	if (flago == vertexCount || flagu == vertexCount) 
	{
		// Polygon is completely outside of screen
		return;
	}

	// Fill polygon
	if (Display::depth == 8)
	{
		const IndexedColorType colorIdx = color.getIndexedColor();
	
		// Render directly to screen memory with 8bpp
		UInt8 *rowBits = (UInt8 *)canvas->drawBits;
		rowBits += ((long)miny * canvas->rowBytes);
		UInt32 colorUInt32 = (((UInt32)colorIdx) << 24) + (((UInt32)colorIdx) << 16) + (((UInt32)colorIdx) << 8) + ((UInt32)colorIdx);
		const UInt16 rowBytes = canvas->rowBytes;

		Int16 *minxptr = &minx[miny];
		Int16 *maxxptr = &maxx[miny];

//		for (int y = miny; y <= maxy; y++)
		for (int y = maxy - miny; y >= 0; y--)
		{
//			int cminx = minx[y];
			int cminx = *minxptr++;
			
			if (cminx == clipxr)
			{
				rowBits += rowBytes;
				continue;
			}
			
//			int cmaxx = maxx[y];
			int cmaxx = *maxxptr++;

			int width = (cmaxx - cminx) + 1;
			
			UInt8 *drawBits = rowBits + cminx;
			
			// Draw short fragments in an ordinary loop
			if (width <= 10)
			{
				while(--width >= 0)
				{
					*drawBits++ = colorIdx;
				}
			}
			else
			{
				// Write the line 4 pixels at a time. Make sure only even memory addresses will be touched.

				switch(cminx & 3)
				{
					case 1:
					*drawBits++ = colorIdx;
					case 2:
					*drawBits++ = colorIdx;
					case 3:
					*drawBits++ = colorIdx;

					width -= 4 - (cminx & 3);
				}
				
// This would be correct, but has been moved into the previous switch clause				
/*				if (cminx & 3)
				{
					width -= 4 - (cminx & 3);
				}
*/
// This would be correct, but is unneccessary because of subsequent shift operation.				
/*				if ((cmaxx + 1) & 3)
				{
					width -= (cmaxx + 1) & 3;
				}
*/
				width >>= 2;
								
				while(--width >= 0)
				{
					(*((UInt32 *)drawBits)) = colorUInt32;					
					drawBits += 4;
				}
								
				switch(cmaxx & 3)
				{
					case 2:
					*drawBits++ = colorIdx;
					case 1:
					*drawBits++ = colorIdx;
					case 0:
					*drawBits++ = colorIdx;
				}
			}
			
			rowBits += rowBytes;
		}
	}
	else   // Not 8bpp
	{
		// Use OS routines (SLOOOOWWW!!!)
		if (canvas->setForegroundColor(color))
		{
			for (int y = miny; y <= maxy; y ++)
			{
				WinDrawLine(minx[y], y, maxx[y], y);
			}
		}
		else
		{
			for (int y = miny; y <= maxy; y ++)
			{
				WinEraseLine(minx[y], y, maxx[y], y);
			}			
		}
	}
}

