/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: FiveWayDigitalController.cpp,v 1.2 2003/05/29 15:44:32 teacy Exp $
 *
 * Copyright (c) 2003 Tilo Christ. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifdef SUPPORT_FIVEWAY
#include <PalmOS.h>
#include <PalmChars.h>

#include "input/FiveWayDigitalController.h"
#include "input/HardKeyManager.h"

using namespace prvrazor;


Boolean FiveWayDigitalController::isSupported()
{
	UInt32 version;
	const Err err = FtrGet(navFtrCreator, navFtrVersion, &version);

	return (err	== errNone);
}


void FiveWayDigitalController::receiveEvent(const EventType*)
{
	// Events are not of interest to this controller.
}



int FiveWayDigitalController::getDirection() const
{
	const UInt32 keyState = HardKeyManager::getCurrentState();

	int result = DigitalController::DIR_RELEASED;

	if (keyState & (keyBitNavLeft | keyBitNavRight | keyBitPageDown | keyBitPageUp))
	{
		if (keyState & keyBitPageDown)
		{
			result |= DigitalController::DIR_DOWN;
		}
		else if (keyState & keyBitPageUp)
		{
			result |= DigitalController::DIR_UP;
		}

		if (keyState & keyBitNavLeft)
		{
			result |= DigitalController::DIR_LEFT;
		}
		else if (keyState & keyBitNavRight)
		{
			result |= DigitalController::DIR_RIGHT;
		}
	}

	return result;
}


int FiveWayDigitalController::getButtonCount() const
{
	return 5;
}


bool FiveWayDigitalController::isButtonPressed(int buttonIndex) const
{
	const UInt32 keyState = HardKeyManager::getCurrentState();

	bool result = false;

	if (buttonIndex == 0)
	{
		result = (keyState & keyBitHard1);
	}
	else if (buttonIndex == 1)
	{
		result = (keyState & keyBitHard2);
	}
	else if (buttonIndex == 2)
	{
		result = (keyState & keyBitHard3);
	}
	else if (buttonIndex == 3)
	{
		result = (keyState & keyBitHard4);
	}
	else if (buttonIndex == 4)
	{
		result = (keyState & keyBitNavSelect);
	}

	return result;
}	


FiveWayDigitalController::FiveWayDigitalController() : DigitalController()
{
}


FiveWayDigitalController::~FiveWayDigitalController()
{
}

#endif
