/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Color.h,v 1.2 2003/03/17 21:19:49 teacy Exp $
 *
 * Copyright (c) 2003 Tilo Christ. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/




#ifndef COLOR_H
#define COLOR_H

#include "RazorSections.h"
#include <PalmOS.h>


/**
 * Representation of a color.
 * <br>
 * Color operations are very sensitive to the current display settings.
 * You should only construct colors while the display is properly setup, and
 * is initialized with the palette that you wish to use.
 * <br>
 * Do not use color objects after you have changed display settings.
 */
class Color
{
	public:
	/**
	 * Construct a color with the specified RGB values.
	 *
	 * @param r the red value from 0-255
	 * @param g the green value from 0-255
	 * @param b the blue value from 0-255
	 */
	Color(UInt8 r, UInt8 g, UInt8 b);

	/**
	 * Set an existing color object to the color with the specified RGB values.
	 * Use this to reuse color objects, which might reduce memory fragmentation.
	 *
	 * @param r the red value from 0-255
	 * @param g the green value from 0-255
	 * @param b the blue value from 0-255
	 */
	void init(UInt8 r, UInt8 g, UInt8 b);


	/**
	 * Get the index of the color in the display's current palette.
	 */
	IndexedColorType getIndexedColor() const;
	RGBColorType getRGBColor() const;

	static const Color BLACK;
	static const Color WHITE;

	private:
	mutable RGBColorType rgbColor;
	mutable Boolean isIndexed;
};


#endif
