/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: HighscoreModel.h,v 1.1 2003/01/26 10:44:08 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#ifndef HIGHSCOREMODEL_H
#define HIGHSCOREMODEL_H

#include <PalmOS.h>
#include "Sections.h"


/**
 * An entry in the highscore table.
 */
class HighscoreEntry
{
	public:
	UInt32 score;
	Char name[16];
};


/**
 * A model and the neccessary logic for highscore management.
 */
class HighscoreModel
{
	/**
	 * For internal usage by highscore model.
	 *
	 * Class that holds all table entries.
	 */
	class HighscoreState
	{
		HighscoreEntry entries[30];
	};

	public:

///@name Construction / Destruction
//@{
	/**
	 * Construct the model from a save-state.
	 */
	HighscoreModel(void *highscoreState);

	/**
	 * Destroy the model.
	 */
	~HighscoreModel();
//@}

	
///@name Logic
//@{
	/**
	 * Return whether the supplied score qualifies as a highscore.
	 */
	Boolean isHighscore(UInt32 score);

	/**
	 * Add the highscore to the highscore table. 
	 * If the number of highscores exceeds the limit, the lowest existing highscore will be discarded.
	 */
	void addHighscore(UInt32 score, Char *name = NULL);
	
	/**
	 * Get the number of highscores in the highscore table.
	 */
	UInt8 getNumHighScores();

	/**
	 * Get the nth entry from the highscore table.
	 */
	HighscoreEntry& getEntry(UInt8 index);	
//@}


///@name Persistence
//@{
	/**
	 * Return a descriptor for a freshly initialized state. 
	 */
	static StateDescriptor *createStateDescriptor();
//@}
};

#endif
