/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Direct8bppBlitter.h,v 1.3 2003/03/17 22:21:23 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef DIRECT_8BPP_BLITTER_H
#define DIRECT_8BPP_BLITTER_H

#include "RazorSections.h"
#include "gfx/Blitter.h"
#include "gfx/Canvas.h"


namespace prvrazor
{

class Direct8bppSolidBitmap
{
	UInt8 *bitmapBits;

	friend class Direct8bppSolidBitmapArray;
};


class Direct8bppSolidBitmapArray : public BitmapArray
{
	public:
	Direct8bppSolidBitmapArray(const Canvas *canvas, UInt16 numBitmaps, UInt16 bitmapID) SEC_RAZOR_INIT;
	virtual void draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;

	protected:
	virtual ~Direct8bppSolidBitmapArray() SEC_RAZOR_INIT;

	private:
	const Direct8bppSolidBitmap *const bitmaps;
	UInt16 bitmapFtrFirstNum;
};


class Direct8bppTransparentBitmap
{
	UInt8 *bitmapBits;
	UInt8 *maskBits;

	friend class Direct8bppTransparentBitmapArray;
};


class Direct8bppTransparentBitmapArray : public BitmapArray
{
	public:
	Direct8bppTransparentBitmapArray(const Canvas *canvas, UInt16 numBitmaps, UInt16 bitmapID, UInt16 maskID) SEC_RAZOR_INIT;
	virtual void draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;

	protected:
	virtual ~Direct8bppTransparentBitmapArray() SEC_RAZOR_INIT;

	private:

#ifdef __MWERKS__
	void drawM68KAsm(
	const void *foregroundBits, const void *backgroundBits, void *drawBits, 
	Int16 yExtent, UInt16 colQuads,
	UInt16 width, UInt16 canvasWidth, Int16 xExtent) const;
#endif

	const Direct8bppTransparentBitmap *const bitmaps;
	UInt16 bitmapFtrFirstNum;
	UInt16 maskFtrFirstNum;
};




class Direct8bppSaveBuffer
{
	UInt8 *bufferBits;
	UInt32 canvasRemainder;
	UInt32 drawOffset;
	UInt16 colQuads;
	Int16 yExtent;
	
	friend class Direct8bppSaveBufferArray;
};


class Direct8bppSaveBufferArray : public SaveBufferArray
{
	public:
    Direct8bppSaveBufferArray(UInt16 numBuffers, Coord width, Coord height) SEC_RAZOR_INIT;
    virtual ~Direct8bppSaveBufferArray() SEC_RAZOR_INIT;
    virtual void save(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;
    virtual void restore(const Canvas *canvas, UInt16 index) const;

    private:

#ifdef __MWERKS__
	void saveM68KAsm(
	void *bufferBits, const void *drawBits,
	Int16 yExtent, UInt16 colQuads, UInt32 canvasRemainder
	) const;

	void restoreM68KAsm(
	const void *bufferBits, void *drawBits,
	Int16 yExtent, UInt16 colQuads, UInt32 canvasRemainder
	) const;
#endif

    Direct8bppSaveBuffer *const buffers;
};




/**
 * Optimized Blitter functionality for 8bpp color display.
 */
class Direct8bppBlitter : public Blitter
{
	public:
	Direct8bppBlitter(Canvas *canvas) SEC_RAZOR_INIT;
	~Direct8bppBlitter() SEC_RAZOR_INIT;

	const BitmapArray *prepareTransparentBitmaps(UInt16 numBitmaps, DmResID bitmapID, DmResID maskID) const SEC_RAZOR_INIT;
	const BitmapArray *prepareSolidBitmaps(UInt16 numBitmaps, DmResID bitmapID) const SEC_RAZOR_INIT;

	SaveBufferArray *prepareSaveBuffers(UInt16 numBuffers, Coord width, Coord height) const SEC_RAZOR_INIT;
	
	void drawText(Color textColor, Color bgColor, FontID font, const Char *chars, Int16 len, Coord x, Coord y) const;	

	void fillRectangle(Color color, Coord topX, Coord topY, Coord bottomX, Coord bottomY) const;
};

}

#endif
