/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: DirectDoubleBuffer.h,v 1.3 2003/05/29 15:46:33 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef DIRECTDOUBLEBUFFER_H
#define DIRECTDOUBLEBUFFER_H

#include "RazorSections.h"
#include "Customization.h"
#include <PalmOS.h>

#include "gfx/Display.h"
#include "gfx/Canvas.h"

namespace prvrazor
{

/**
 * DirectDoubleBuffer achieves double buffering by directly accessing screen memory for maximum speed.
 *
 * TODO:
 * - Are there hardware accelerated devices where the OS provides more speed?
 */
class DirectDoubleBuffer
{
	public:

	/** 
	 * Return true, if using a DirectDoubleBuffer is possible, 
	 * i.e. we know enough about the device to feel comfortable hacking on its hardware.
	 */
	static Boolean isSupported(const Canvas *targetCanvas) SEC_RAZOR_INIT;

	// TODO: Should there be a rectangle param, rather than a viewport? Is viewport parameter even neccessary?
	DirectDoubleBuffer(const Canvas *sourceCanvas, const Canvas *targetCanvas, const ViewportProperties& viewport) SEC_RAZOR_INIT;
	
	~DirectDoubleBuffer() SEC_RAZOR_INIT;

	void prepareCopy(RectangleType &copyBounds);
	void copy();

	private:
	void *displayBits;	
	void *viewportBits;
	UInt32 yOffsetBytes;
	Coord viewportExtentY;
    Coord targetWidth;

	Coord copyTop;
	Coord copyExtent;

	/// Can the copy() operation be skipped
	Boolean dontCopy;

	/// Make smart copy of screen memory.
	void copyScreen();

#ifdef __MWERKS__  
	/// Copy packets of screen memory with optimized M68K assembler (1 packet == PACKETSIZE bytes).
	void copyScreenLinesM68KAsm(const void *fromPtr, void *toPtr, UInt16 packetCount);
#endif

	UInt16    screenLineSize; 

	// One "copy unit" is the smallest amount of vertical screen space that can be copied,
	// i.e. the finest possible granularity of the copy operation.
	UInt16    packetsPerCopyUnit;
	UInt16    linesPerCopyUnitShift;
	UInt16    linesPerCopyUnitMask;
	
	static const UInt16 PACKETSIZE = 160;
};

}

#endif
