/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: PalmOSBlitter.h,v 1.3 2003/03/17 22:21:23 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef PALMOS_BLITTER_H
#define PALMOS_BLITTER_H

#include "RazorSections.h"
#include "gfx/Blitter.h"
#include "gfx/Canvas.h"


namespace prvrazor
{


class PalmOSTransparentBitmap
{
	WinHandle bitmap;
	WinHandle mask;

	friend class PalmOSTransparentBitmapArray;
};


class PalmOSTransparentBitmapArray : public BitmapArray
{
	public:
	PalmOSTransparentBitmapArray(const Canvas *canvas, UInt16 numBitmaps, UInt16 bitmapID, UInt16 maskID) SEC_RAZOR_INIT;
	virtual void draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;

	protected:
	virtual ~PalmOSTransparentBitmapArray() SEC_RAZOR_INIT;

	private:
	const PalmOSTransparentBitmap *const bitmaps;
};



class PalmOSSolidBitmap
{
	BitmapType *bitmap;
	MemHandle bitmapResH;

	friend class PalmOSSolidBitmapArray;
};


class PalmOSSolidBitmapArray : public BitmapArray
{
	public:
	PalmOSSolidBitmapArray(const Canvas *canvas, UInt16 numBitmaps, UInt16 bitmapID) SEC_RAZOR_INIT;
	virtual void draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;

	protected:
	virtual ~PalmOSSolidBitmapArray() SEC_RAZOR_INIT;

	private:
	const PalmOSSolidBitmap *const bitmaps;
};



class PalmOSSaveBuffer
{
	WinHandle buffer;
	Coord x;
	Coord y;
	
	friend class PalmOSSaveBufferArray;
};


class PalmOSSaveBufferArray : public SaveBufferArray
{
	public:
    PalmOSSaveBufferArray(UInt16 numBuffers, Coord width, Coord height) SEC_RAZOR_INIT;
    virtual ~PalmOSSaveBufferArray() SEC_RAZOR_INIT;
    virtual void save(const Canvas *canvas, UInt16 index, Coord x, Coord y) const;
    virtual void restore(const Canvas *canvas, UInt16 index) const;

    private:
    PalmOSSaveBuffer *const buffers;
};


/**
 * Perfectly pure implementation of the Blitter functionality.
 */
class PalmOSBlitter : public Blitter
{
	public:
	PalmOSBlitter(Canvas *canvas) SEC_RAZOR_INIT;
	~PalmOSBlitter() SEC_RAZOR_INIT;

	const BitmapArray *prepareTransparentBitmaps(UInt16 numBitmaps, DmResID bitmapID, DmResID maskID) const SEC_RAZOR_INIT;
	const BitmapArray *prepareSolidBitmaps(UInt16 numBitmaps, DmResID bitmapID) const SEC_RAZOR_INIT;

	SaveBufferArray *prepareSaveBuffers(UInt16 numBuffers, Coord width, Coord height) const SEC_RAZOR_INIT;

	void drawText(Color textColor, Color bgColor, FontID font, const Char *chars, Int16 len, Coord x, Coord y) const;	

	void fillRectangle(Color color, Coord topX, Coord topY, Coord bottomX, Coord bottomY) const;
};


}

#endif
