/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: SonyClieDisplay.h,v 1.3 2003/05/29 15:46:32 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef SONYCLIE_DISPLAY_H
#define SONYCLIE_DISPLAY_H

#include "RazorSections.h"

#ifdef SUPPORT_CLIE
#include <SonyCLIE.h>
#include "gfx/Display.h"
#ifndef MODE_PURITY
#include "gfx/DirectDoubleBuffer.h"
#endif


namespace prvrazor
{


/**
 * A Display that supports special features in Sony Clie devices.
 * The Clie devices need special handling for hires/lo-res switching.
 * They also have speed characteristics that differ from those of 
 * other Palm devices.
 */
class SonyClieDisplay : public Display
{
	public:

	SonyClieDisplay() SEC_RAZOR_INIT;
	
	~SonyClieDisplay() SEC_RAZOR_INIT;

	static Boolean isSupported() SEC_RAZOR_INIT;

	void beginDrawViewport();
		
	void endDrawViewport(RectangleType *bounds);

	void showViewport();

	void captureDisplay() SEC_RAZOR_INIT;
	void releaseDisplay() SEC_RAZOR_INIT;

    void initDisplay(const DisplayProperties& displayProperties) SEC_RAZOR_INIT;
	void initViewport(const ViewportProperties& viewport) SEC_RAZOR_INIT;

	protected:
	void createViewportCanvas(const ViewportProperties& viewport) SEC_RAZOR_INIT;
	void resetDisplay() SEC_RAZOR_INIT;

	private:
    void initPhysicalDisplay() SEC_RAZOR_INIT;
	static Boolean openSonyHRLib() SEC_RAZOR_INIT;
	static void closeSonyHRLib() SEC_RAZOR_INIT;

	static UInt16 sonyHRLibRefNum;
};

}

#endif

#endif
