/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: HardKeyManager.h,v 1.2 2003/02/15 12:09:15 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef HARDKEYMANAGER_H
#define HARDKEYMANAGER_H

#include "Customization.h"
#include <PalmOS.h>
#ifdef SUPPORT_FIVEWAY
#include <PalmChars.h>
#endif


namespace prvrazor
{

/**
 * HardKeyManager manages the hard keys (Up/Down, Calendar, Address Book, etc.) of the device.
 * It can prevent them from switching apps, it can modify their repeat rate, and it can poll their state.
 */
class HardKeyManager
{
	public:	
	/**
	 * Prepare the HardKeyManager for use.
	 */
	static void init();
	
	
	/**
	 * Release the hard keys and all other resources.
	 */
	static void destroy();

	
	/**
	 * Prepare the hard keys for polling (i.e. turn off event generation). Idempotent.
	 */
	static void captureHardKeys();
	

	/**
	 * Return the hard keys to event generation mode. Idempotent.
	 */
	static void releaseHardKeys();
		

	/**
	 * Get the current state of the hard keys.
	 * The result will be identical to that from
	 * the KeyCurrentState() PalmOS call.
	 *
	 * @return the currently pressed keys. Use the keyXXX constants from KeyMgr.h to decipher it.
	 */
	static UInt32 getCurrentState();
	

	/**
	 * Express your interest in the keys specified by the mask.
	 *
	 * @param the keys you want to listen to. Use the keyXXX constants from KeyMgr.h.
	 */
	static void setMask(UInt32 mask);


	private:

	// HardKeyManager cannot be instantiated or destroyed.
	HardKeyManager() {}
	~HardKeyManager() {}



	// Constants
#ifdef SUPPORT_FIVEWAY
	static const UInt32 keysAllowedMask = (keyBitHard1 | keyBitHard2 | keyBitHard3 | keyBitHard4 | keyBitPageUp | keyBitPageDown | keyBitNavLeft | keyBitNavRight | keyBitNavSelect );
#else
	static const UInt32 keysAllowedMask = (keyBitHard1 | keyBitHard2 | keyBitHard3 | keyBitHard4 | keyBitPageUp | keyBitPageDown );
#endif
	
	// Current key settings
	static UInt32 mask;

	// Original key settings
	static UInt16 oldInitDelay;
	static UInt16 oldPeriod;
	static UInt16 oldDoubleTapDelay;
	static Boolean oldQueueAhead;
};

}

#endif
